@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# $Id: setup.py 11562 2019-05-17 19:18:19Z Lavender $
+# $Id: setup.py 11821 2019-12-26 02:47:12Z Lavender $
 #
 # Copyright (c) 2017 Nuwa Information Co., Ltd, All Rights Reserved.
 #
@@ -12,8 +12,8 @@
 # limitations under the License.
 #
 # $Author: Lavender $
-# $Date: 2019-05-18 04:18:19 +0900 (週六, 18 五月 2019) $
-# $Revision: 11562 $
+# $Date: 2019-12-26 11:47:12 +0900 (週四, 26 十二月 2019) $
+# $Revision: 11821 $
 
 import os
 import sys
@@ -25,6 +25,7 @@ import argparse
 import locale
 import hashlib
 import traceback
+import threading
 import pkg_resources
 
 try:
@@ -40,6 +41,12 @@ try:
 except Exception as e:
     os.system("pip install termcolor==1.1.0")
     from termcolor import colored, cprint
+    
+try:
+    import requests
+except Exception as e:
+    os.system("pip install requests")
+    import requests
 
 logger = logging.getLogger(__name__)
 logger.setLevel(logging.DEBUG)
@@ -79,9 +86,6 @@ CKEDITOR_SETTINGS = {
 }
 '''
 
-SCHEMA_ORIGIN_MD5 = "c27ea4f5a250178b0a417b719c40c3a8"
-SCHEMA_FIX_MD5 = "08fc81f9012c4a7f94782edc030885e7"
-
 SCHEMA_FIX = '''
     def __enter__(self):
         # Some SQLite schema alterations need foreign key constraints to be
@@ -96,7 +100,9 @@ SCHEMA_FIX = '''
         self.connection.enable_constraint_checking()
 '''
 
-SCHEMA_ORIGIN = '''
+SCHEMA_FIX_MD5_1_11 = "08fc81f9012c4a7f94782edc030885e7"
+SCHEMA_ORIGIN_MD5_DJANGO_1_11 = "c27ea4f5a250178b0a417b719c40c3a8"
+SCHEMA_ORIGIN_DJANGO_1_11 = '''
     def __enter__(self):
         with self.connection.cursor() as c:
             # Some SQLite schema alterations need foreign key constraints to be
@@ -115,6 +121,47 @@ SCHEMA_ORIGIN = '''
             c.execute('PRAGMA foreign_keys = %s' % int(self._initial_pragma_fk))
 '''
 
+SCHEMA_FIX_MD5_2_1 = "de8e3a3770fdb73b3f4478e0410ceaf9"
+SCHEMA_ORIGIN_MD5_DJANGO_2_1 = "280dba9301b0a827962786c34456ca54"
+SCHEMA_ORIGIN_DJANGO_2_1 = '''
+    def __enter__(self):
+        # Some SQLite schema alterations need foreign key constraints to be
+        # disabled. Enforce it here for the duration of the schema edition.
+        if not self.connection.disable_constraint_checking():
+            raise NotSupportedError(
+                'SQLite schema editor cannot be used while foreign key '
+                'constraint checks are enabled. Make sure to disable them '
+                'before entering a transaction.atomic() context because '
+                'SQLite3 does not support disabling them in the middle of '
+                'a multi-statement transaction.'
+            )
+        self.connection.cursor().execute('PRAGMA legacy_alter_table = ON')
+        return super().__enter__()
+
+    def __exit__(self, exc_type, exc_value, traceback):
+        super().__exit__(exc_type, exc_value, traceback)
+        self.connection.cursor().execute('PRAGMA legacy_alter_table = OFF')
+        self.connection.enable_constraint_checking()
+'''
+
+LOG_URL = "https://www.django-cms-themes.com/log/"
+
+def sendLog(data):
+    try:
+        def job():
+            requests.post(LOG_URL, data=data)
+        
+        t = threading.Thread(target=job)
+        t.start()
+    except Exception as e:
+        # do nothing
+        pass
+
+if not sys.version_info.major == 3:
+    logger.warning(
+        colored("Python 2.7 support will be removed next version"
+                ", please consider upgrade to Python 3", "yellow"))
+
 def copyToPath(baseDir, templateDir, staticDir, mode):
     if not templateDir:
         templateDir = os.path.join(baseDir, 'templates')
@@ -182,7 +229,7 @@ def canAddPlugin(pkgName, pluginName):
         return True
                
 def writeSettings(settingsPath, templateDir, staticDir, hasCmsTemplate, baseDir,
-                  wizard=False, createProject=False):
+                  cmsVersion, wizard=False, createProject=False):
     logger.info(
         colored("-----Start write and backup settings.py-----", "yellow"))
     def getCMSTemplates():
@@ -242,13 +289,10 @@ else:
     STATIC_ROOT = os.path.join(BASE_DIR, 'static')
 '''
         import cms
-    
-        if cms.__version__.startswith('3.5'):
-            pluginList = []
-            requirementList = []
-        else:
-            pluginList = []
-            requirementList = []
+        import django
+
+        pluginList = []
+        requirementList = []
         
         if createProject:
             if wizard:
@@ -264,27 +308,17 @@ else:
                 yesOrNo = 'yes'
                 
             if yesOrNo == 'yes':
-                if cms.__version__.startswith('3.4'):
-                    if canAddPlugin(
-                        'djangocms-file', 'djangocms_file==2.0.2'):
-                        pluginList.append('djangocms_file')
-                        requirementList.append('djangocms_file==2.0.2')
-                        
-                    if canAddPlugin(
-                        'djangocms-picture', 'djangocms_picture==2.0.6'):
-                        pluginList.append('djangocms_picture')
-                        requirementList.append('djangocms_picture==2.0.6')
-                        
+                
                 if canAddPlugin(
                     'djangocms-history', 'djangocms_history==0.5.3'):
                     pluginList.append('djangocms_history')
                     requirementList.append('djangocms_history==0.5.3')
-                if canAddPlugin(
-                    'djangocms-forms', 'djangocms_forms==0.2.5'):
-                    pluginList.append('djangocms_forms')
-                    requirementList.append('djangocms_forms==0.2.5')
                     
-            requirementList.append('djangocms_text_ckeditor==3.5.3')
+                if not django.VERSION >= (2, 0):
+                    if canAddPlugin(
+                        'djangocms-forms', 'djangocms_forms==0.2.5'):
+                        pluginList.append('djangocms_forms')
+                        requirementList.append('djangocms_forms==0.2.5')
             
             with open(os.path.join(baseDir, "requirements.txt"), 'a') as reqs:
                 reqs.write("\n")
@@ -293,6 +327,11 @@ else:
                    
         content += "INSTALLED_APPS += %s" % str(tuple(pluginList))
         content += CKEDITOR_SETTINGS
+        
+        if int(cmsVersion[0]) == 3 and int(cmsVersion[1]) == 7:
+            content = content.replace(
+                "'djangocms_bootstrap4.contrib.bootstrap4_picture',", 
+                "#'djangocms_bootstrap4.contrib.bootstrap4_picture',")
         settings.write(content)
            
                
@@ -333,28 +372,51 @@ def loadData(baseDir, language):
     return True
     
 def fixSchema(schemaPath):
+    import django 
     try:
         with open(schemaPath, 'r') as f:
             content = f.read()
+            
+        if not django.VERSION >= (2, 0):
+            SCHEMA_FIX_MD5 = SCHEMA_FIX_MD5_1_11
+            SCHEMA_ORIGIN = SCHEMA_ORIGIN_DJANGO_1_11 
+            SCHEMA_ORIGIN_MD5 = SCHEMA_ORIGIN_MD5_DJANGO_1_11
+        else:
+            SCHEMA_FIX_MD5 = SCHEMA_FIX_MD5_2_1
+            SCHEMA_ORIGIN = SCHEMA_ORIGIN_DJANGO_2_1
+            SCHEMA_ORIGIN_MD5 = SCHEMA_ORIGIN_MD5_DJANGO_2_1
         
         if hashlib.md5(content.encode()).hexdigest() == SCHEMA_ORIGIN_MD5:
             content = content.replace(SCHEMA_ORIGIN, SCHEMA_FIX)
+              
+        with open(schemaPath, 'w') as f:
+            f.write(content)
             
-            with open(schemaPath, 'w') as f:
-                f.write(content)
+        with open(schemaPath, 'r') as f:
+            content = f.read()
     except Exception as e:
         pass
     
 def revertSchema(schemaPath):
+    import django 
     try:
         with open(schemaPath, 'r') as f:
             content = f.read()
             
+        if not django.VERSION >= (2, 0):
+            SCHEMA_FIX_MD5 = SCHEMA_FIX_MD5_1_11
+            SCHEMA_ORIGIN = SCHEMA_ORIGIN_DJANGO_1_11 
+            SCHEMA_ORIGIN_MD5 = SCHEMA_ORIGIN_MD5_DJANGO_1_11
+        else:
+            SCHEMA_FIX_MD5 = SCHEMA_FIX_MD5_2_1
+            SCHEMA_ORIGIN = SCHEMA_ORIGIN_DJANGO_2_1
+            SCHEMA_ORIGIN_MD5 = SCHEMA_ORIGIN_MD5_DJANGO_2_1
+            
         if hashlib.md5(content.encode()).hexdigest() == SCHEMA_FIX_MD5:
             content = content.replace(SCHEMA_FIX, SCHEMA_ORIGIN)
             
-            with open(schemaPath, 'w') as f:
-                f.write(content)
+        with open(schemaPath, 'w') as f:
+            f.write(content)
     except Exception as e:
         pass
        
@@ -366,6 +428,18 @@ if __name__ == "__main__":
             type=str,  nargs='?', default=".",
             help="Path to project, default is '.'"
         )
+        
+        parser.add_argument(
+            "--djangoVersion", metavar='djangoVersion', 
+            type=str,  nargs='?', default="2.2",
+            help="Django version, default is '2.2'"
+        )
+        
+        parser.add_argument(
+            "--projectName", metavar='projectName', 
+            type=str,  nargs='?',
+            help="Django project name."
+        )
            
         parser.add_argument(
             '-w', '--wizard', 
@@ -374,6 +448,42 @@ if __name__ == "__main__":
        
         args = parser.parse_args()
         
+        try:
+            if os.path.isfile(os.path.join(SETUP_DIR, 'MANIFAST.INFO')):
+                with open(os.path.join(SETUP_DIR, 'MANIFAST.INFO')) as f:
+                    data = json.loads(f.read())
+                upc = data['upc']
+            else:
+                upc = None
+        except Exception as e:
+            upc = None
+            
+        projectName = args.projectName
+        djangoVersion = args.djangoVersion
+        
+        djangoVersionTable = {
+            '2.2': '<3.0.0,>=2.2',
+            '1.11': '<2.0.0,>=1.11',
+        }
+        if not djangoVersion in djangoVersionTable:
+            logger.warning(colored(
+                "Django version must be 1.11 or 2.2", "yellow"))
+            sys.exit(1)
+        else:
+            djangoVersion = djangoVersionTable[djangoVersion]
+
+        try:
+            os.system("pip install -U \"django%s\"" % djangoVersion)
+        except Exception as e:
+            logger.error(
+                colored("Can't install django%s" % djangoVersion, 
+                        "red"))
+            sys.exit(1)
+            
+        import django
+        djangoVersion = "%d.%d.%d" % (
+            django.VERSION[0], django.VERSION[1], django.VERSION[2],)
+        
         # modify django schema.py
         # issue: https://github.com/django/django/pull/10733/commits/c8ffdbe514b55ff5c9a2b8cb8bbdf2d3978c188f
         try:
@@ -395,45 +505,51 @@ if __name__ == "__main__":
                 
             cmsVersion = data["DjangoCMSVersion"]
         else:
-            cmsVersion = '3.5.1'
+            cmsVersion = '3.7.0'
+            
+        # default
+        cmsVersion = '3.7.0'
         
-        if cmsVersion.startswith("3.4"):
-            installVersion = "0.9.7"
-        elif cmsVersion.startswith("3.5"):
-            installVersion = "1.0.0"
-        elif cmsVersion.startswith("3.6"):
-            installVersion = "1.1.0"
+        if cmsVersion.startswith("3.6"):
+            installVersion = "1.2.0"
+        elif cmsVersion.startswith("3.7"):
+            installVersion = "1.2.0"
         else:
-            installVersion = "1.1.0"
+            installVersion = "1.2.0"
+            
+        sendLog({
+            "level": "INFO",
+            "message": "User setup template",
+            "djangoVersion": djangoVersion,
+            "cmsVersion": str(cmsVersion),
+            "python": str(sys.version_info),
+            "upc": upc,
+        })
            
         path = args.path
-       
+        
         settingsPath = findSettings(path)
         
         createProject = False
        
         if not settingsPath:
-            if sys.version_info.major == 3:
-                print()
-                projectName = input(
-                    ("Django project not found. "
-                     "Auto create one by djangocms-installer==%s."
-                     "\nPlease input a project name:") % installVersion)
-            else:
-                print
-                projectName = raw_input(
-                    ("Django project not found. "
-                     "Auto create one by djangocms-installer==%s."
-                     "\nPlease input a project name:") % installVersion)
+            if not projectName:
+                if sys.version_info.major == 3:
+                    print()
+                    projectName = input(
+                        ("Django project not found. "
+                         "Auto create one by djangocms-installer==%s."
+                         "\nPlease input a project name:") % installVersion)
+                else:
+                    print
+                    projectName = raw_input(
+                        ("Django project not found. "
+                         "Auto create one by djangocms-installer==%s."
+                         "\nPlease input a project name:") % installVersion)
             
             result = os.system(
                 "pip install djangocms-installer==%s" % installVersion)
                 
-            try:
-                import django
-            except Exception as e:
-                os.system("pip install django==1.11.20")
-                
             if schemaPath:
                 # modify django schema.py
                 logger.warning(colored(
@@ -464,6 +580,15 @@ if __name__ == "__main__":
             if local and local.upper() in ["ZH_TW", "ZH_HANT"]:
                 command = "%s --languages=zh-hant" % command
                 
+            if djangoVersion:
+                command = "%s --django-version=%s" % (
+                    command, ".".join(djangoVersion.split(".")[:2]))
+                
+            if cmsVersion:
+                command = "%s --cms-version=%s" % (
+                    command, ".".join(cmsVersion.split(".")[:2]))
+                
+            logger.info(colored(command, "yellow"))
             result = os.system(command)
            
             createProject = True
@@ -485,18 +610,14 @@ if __name__ == "__main__":
         settingModule = settingModule.replace("\\", ".")
        
         sys.path.append(path)
-        os.environ['DJANGO_SETTINGS_MODULE'] = settingModule
-       
-        import django
-       
+        os.environ['DJANGO_SETTINGS_MODULE'] = settingModule        
+            
         try:
             import cms
         except Exception as e:
             os.system("pip install django-cms==%s" % cmsVersion)
             import cms
 
-        import cms
-        
         if cms.__version__.startswith('3.5'):
             try:
                 import cmsplugin_filer_image
@@ -504,34 +625,32 @@ if __name__ == "__main__":
                 os.system("pip install cmsplugin-filer==1.1.3")
                 import cms
                 
-        import djangocms_text_ckeditor
-        if not djangocms_text_ckeditor.__version__.startswith("3.5"):
+        import django
+         
+        if django.VERSION <= (2, 0): 
             try:
-                os.system("pip install -U djangocms_text_ckeditor==3.5.3")
+                os.system(
+                    "pip install -U djangocms_text_ckeditor==3.5.3 --no-deps")
+                os.system(
+                    "pip install -U html5lib==0.9999999 --no-deps")
+                os.system("pip install cmsplugin-filer==1.1.3")
             except Exception as e:
                 logger.error(
                     colored("Can't install djangocms_text_ckeditor==3.5.3", 
                             "red"))
                 sys.exit(1)
+        else:
+            try:
+                os.system(
+                    "pip install -U djangocms_text_ckeditor==3.8.0 --no-deps")
+            except Exception as e:
+                logger.error(
+                    colored("Can't install djangocms_text_ckeditor==3.8.0", 
+                            "red"))
+                sys.exit(1)
            
         cmsVersion = cms.__version__.split('.')
        
-        cms34 = True
-        try:
-            if int(cmsVersion[0]) <= 3 and int(cmsVersion[1]) <= 4:
-                cms34 = True
-            else:
-                cms34 = False
-        except Exception as e:
-            cms34 = False
-           
-        if cms34:
-            if not (django.VERSION[0] == 1 and django.VERSION[1] <= 10):
-                logger.error(colored(
-                    "Your installed Django above 1.10 which is not supported by "
-                    "Django CMS 3.4 yet, please reinstall Django or you may "
-                    "encounter some issues while using Django CMS.", "red"))
-                sys.exit(1)
         django.setup()
        
         from django.conf import settings
@@ -584,7 +703,7 @@ if __name__ == "__main__":
             hasCmsTemplate = False
            
         writeSettings(settingsPath, 
-            templateDir, staticDir, hasCmsTemplate, baseDir,
+            templateDir, staticDir, hasCmsTemplate, baseDir, cmsVersion,
             wizard=wizard, createProject=createProject)
            
         os.chdir(baseDir)
@@ -631,6 +750,15 @@ if __name__ == "__main__":
     except Exception as e:
         print("")
         logger.error(colored("ERROR MESSAGE:", "red"))
+        sendLog({
+            "level": "ERROR",
+            "error": traceback.format_exc(),
+            "exception": str(e),
+            "djangoVersion": djangoVersion,
+            "cmsVersion": str(cmsVersion),
+            "python": str(sys.version_info),
+            "upc": upc,
+        })
         traceback.print_exc()
         print("")
         logger.error(colored(
